@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1999-2002, 2004-2006 Free Software Foundation, Inc.
+ * Copyright (C) 1999-2002, 2004-2009 Free Software Foundation, Inc.
  * This file is part of the GNU LIBICONV Library.
  *
  * The GNU LIBICONV Library is free software; you can redistribute it
@@ -51,10 +51,16 @@ struct mbtowc_funcs {
    */
 };
 
+/* Return code if invalid input after a shift sequence of n bytes was read.
+   (xxx_mbtowc) */
+#define RET_SHIFT_ILSEQ(n)  (-1-2*(n))
 /* Return code if invalid. (xxx_mbtowc) */
-#define RET_ILSEQ      -1
+#define RET_ILSEQ           RET_SHIFT_ILSEQ(0)
 /* Return code if only a shift sequence of n bytes was read. (xxx_mbtowc) */
-#define RET_TOOFEW(n)  (-2-(n))
+#define RET_TOOFEW(n)       (-2-2*(n))
+/* Retrieve the n from the encoded RET_... value. */
+#define DECODE_SHIFT_ILSEQ(r)  ((unsigned int)(RET_SHIFT_ILSEQ(0) - (r)) / 2)
+#define DECODE_TOOFEW(r)       ((unsigned int)(RET_TOOFEW(0) - (r)) / 2)
 
 /*
  * Data type for conversion unicode -> multibyte
@@ -163,6 +169,7 @@ struct conv_struct {
 #include "cp850.h"
 #include "cp862.h"
 #include "cp866.h"
+#include "cp1131.h"
 #include "mac_roman.h"
 #include "mac_centraleurope.h"
 #include "mac_iceland.h"
@@ -182,6 +189,7 @@ struct conv_struct {
 #include "georgian_ps.h"
 #include "koi8_t.h"
 #include "pt154.h"
+#include "rk1048.h"
 #include "mulelao.h"
 #include "cp1133.h"
 #include "tis620.h"
